package com.pancik.libgdx.zumpatalentov;

import com.badlogic.gdx.Gdx;
import com.badlogic.gdx.audio.Music;
import com.badlogic.gdx.audio.Sound;
import com.pancik.libgdx.zumpatalentov.util.RandomGenerator;
import java.util.HashMap;
import java.util.Map;

public final class SoundData {
    public static boolean SOUND_IS_ON = true;
    
    public static Map<String, Sound> sounds;
    public static Map<String, Music> musics;
    
    public static void loadData() {
        sounds = new HashMap<String, Sound>();
        musics = new HashMap<String, Music>();
        
        musics.put("grass_birds", Gdx.audio.newMusic(Gdx.files.internal("data/sound/grass_and_birds_2.mp3")));
        musics.put("main_theme", Gdx.audio.newMusic(Gdx.files.internal("data/sound/main_theme.mp3")));
        
        sounds.put("tofu_squash", Gdx.audio.newSound(Gdx.files.internal("data/sound/currency/tofu.mp3")));
        
        sounds.put("axe_clash", Gdx.audio.newSound(Gdx.files.internal("data/sound/battle/axe_clash.mp3")));
        sounds.put("electric_zap", Gdx.audio.newSound(Gdx.files.internal("data/sound/battle/electric_zap.mp3")));
        sounds.put("laser_pistol", Gdx.audio.newSound(Gdx.files.internal("data/sound/battle/laser_pistol.mp3")));
        sounds.put("laser_rifle", Gdx.audio.newSound(Gdx.files.internal("data/sound/battle/laser_rifle.mp3")));
        sounds.put("punch", Gdx.audio.newSound(Gdx.files.internal("data/sound/battle/punch.mp3")));
        
        sounds.put("explosion", Gdx.audio.newSound(Gdx.files.internal("data/sound/explosions/explosion_small.mp3")));
        
        sounds.put("guts_1", Gdx.audio.newSound(Gdx.files.internal("data/sound/battle/guts_1.mp3")));
        sounds.put("guts_2", Gdx.audio.newSound(Gdx.files.internal("data/sound/battle/guts_2.mp3")));
        
        sounds.put("coins_1", Gdx.audio.newSound(Gdx.files.internal("data/sound/currency/coins_1.mp3")));
        sounds.put("coins_2", Gdx.audio.newSound(Gdx.files.internal("data/sound/currency/coins_2.mp3")));
        sounds.put("work_1", Gdx.audio.newSound(Gdx.files.internal("data/sound/poof_1.mp3")));
        sounds.put("work_2", Gdx.audio.newSound(Gdx.files.internal("data/sound/poof_2.mp3")));
    }
    
    public static void disposeData() {
        for(Map.Entry<String, Sound> sound : sounds.entrySet()) {
            sound.getValue().dispose();
        }
        
        for(Map.Entry<String, Music> music : musics.entrySet()) {
            music.getValue().dispose();
        }
    }
    
    public static void playSound(String name) {
        if(SOUND_IS_ON) {
            SoundData.sounds.get(name).play();
        }
    }
    
    public static void playSound(String name, float volume) {
        if(SOUND_IS_ON) {
            SoundData.sounds.get(name).play(volume);
        }
    }
    
    public static void playRandomSound(String ... names) {
        playSound(names[RandomGenerator.getRandom(names.length)]);
    }
    
    public static void playMusic(String name) {
        SoundData.musics.get(name).play();
    }
    
    public static void playMenuMusic() {
        stopEngineMusic();
        if(SOUND_IS_ON) {
            if(!SoundData.musics.get("main_theme").isLooping()) {
                SoundData.musics.get("main_theme").setLooping(true);
                SoundData.playMusic("main_theme");
            }
        }
    }
    
    public static void stopMenuMusic() {
        SoundData.musics.get("main_theme").pause();
    }
    
    public static void playEngineMusic() {
        stopMenuMusic();
        if(SOUND_IS_ON) {
            SoundData.musics.get("grass_birds").setLooping(true);
            SoundData.playMusic("grass_birds");
        }
    }
    
    public static void stopEngineMusic() {
        SoundData.musics.get("grass_birds").stop();
    }
}
