package com.pancik.libgdx.zumpatalentov.engine.building;

import com.badlogic.gdx.graphics.Texture;
import com.badlogic.gdx.math.Vector2;
import com.pancik.libgdx.zumpatalentov.DrawableData;
import com.pancik.libgdx.zumpatalentov.engine.Engine;
import com.pancik.libgdx.zumpatalentov.engine.player.PlayablePlayer;
import com.pancik.libgdx.zumpatalentov.engine.player.Player;
import com.pancik.libgdx.zumpatalentov.entity.attackable.AttackableEntity;
import com.pancik.libgdx.zumpatalentov.entity.unit.FrankieUnit;
import com.pancik.libgdx.zumpatalentov.entity.unit.UnitEntity;
import com.pancik.libgdx.zumpatalentov.entity.unit.buff.Buff;
import com.pancik.libgdx.zumpatalentov.util.MouseEvent;
import com.pancik.libgdx.zumpatalentov.util.RenderUtils;

public class FrankieFactoryBuilding extends BaseBuilding{
    public static Texture FRANKIE_FACTORY_TEXTURE = DrawableData.textures.get("frankie_factory");

    private static final int TEX_SIZE_X = 512;
    private static final int TEX_SIZE_Y = 512;
    private static final int[] ANIMATION_STEPS = {0,0,0,0,0,0,0,0, 1,1,1,1,1,1,1,1, 2,2,2,2,2,2,2,2, 3,3,3,3,3,3,3,3};
    
    private static final float DEFAULT_SIZE_X = 192.0f;
    private static final float DEFAULT_SIZE_Y = 192.0f;
    
    private static final int MAX_HEALTH = 300;
    private static final int FRANKIE_CREATION_TIME = 600;
    private static final int INCOME_PER_MINUTE = 20;
    
    private Engine.EngineControls engineControls;
    
    private int animationStep = 0;
    private int creationStep = 0;
    
    private int health = MAX_HEALTH;
    
    public FrankieFactoryBuilding(Vector2 pos, Vector2 size, int team, PlayablePlayer owner, Engine.EngineControls engineControls) {
        super(pos, size, team, owner);
        this.engineControls = engineControls;
        
        this.owner.getResourceManager().addIncomePerMinute(INCOME_PER_MINUTE);
    }
    
    @Override
    public void getHit(int damage, AttackableEntity from) {
        if(!isDead()) {
            health -= damage;
            
            PlayablePlayer playableOwner = (PlayablePlayer)owner;
            playableOwner.onBaseBuildingHit(health, MAX_HEALTH);
        }
    }

    @Override
    public boolean isDead() {
        return (health <= 0);
    }

    @Override
    public void tick() {
        if(!isDead()) {
            creationStep++;
            if(creationStep >= (FRANKIE_CREATION_TIME * owner.getUpgradeCoefficient(Buff.PercentageStat.SPAWN_SPEED))) {
                UnitEntity newFrankie = new FrankieUnit(owner.getSpawnPosition(), FrankieUnit.getDefaultSize(), team, owner, engineControls);
                newFrankie.setWaypointPath(owner.getRandomWaypointPath());
                engineControls.addEntity(newFrankie);
                
                creationStep = 0;
            }
            
            animationStep++;
            if(animationStep >= ANIMATION_STEPS.length) {
                animationStep = 0;
            }
        }
    }

    @Override
    public void render() {
        if(!isDead()) {
            RenderUtils.blit(DrawableData.spriteBatch,
                    FRANKIE_FACTORY_TEXTURE,
                    new RenderUtils.ScreenInfo(pos.x - size.x/2, pos.y - size.y/2, size.x, size.y),
                    new RenderUtils.ScreenInfo(ANIMATION_STEPS[animationStep] * TEX_SIZE_X, 0, TEX_SIZE_X, TEX_SIZE_Y));
        }
    }

    @Override
    public boolean remove() {
        return isDead();
    }

    @Override
    public void onRemove() {
        this.owner.getResourceManager().addIncomePerMinute(-INCOME_PER_MINUTE);
    }
    
    public static Vector2 getDefaultSize() {
        return new Vector2(DEFAULT_SIZE_X, DEFAULT_SIZE_Y);
    }

    @Override
    public boolean handle(MouseEvent me) {
        return false;
    }

    @Override
    public void forceRemove() {
        health = 0;
    }

    @Override
    public boolean hitsBuilding(int x, int y) {
        if(x >= pos.x - size.x/2 && x <= pos.x + size.x/2 && y >= pos.y - size.y/2 && y <= pos.y + size.y/2) {
            return true;
        } else {
            return false;
        }
    }
    
    @Override
    public boolean showSellMenu() {
        return true;
    }
}
