package com.pancik.libgdx.zumpatalentov.engine.building;

import com.badlogic.gdx.Gdx;
import com.badlogic.gdx.graphics.Texture;
import com.badlogic.gdx.math.Vector2;
import com.badlogic.gdx.math.Vector3;
import com.pancik.libgdx.zumpatalentov.DrawableData;
import com.pancik.libgdx.zumpatalentov.engine.Engine;
import com.pancik.libgdx.zumpatalentov.engine.particle.BloodExplosionParticle;
import com.pancik.libgdx.zumpatalentov.engine.particle.SmallExplosion;
import com.pancik.libgdx.zumpatalentov.engine.player.PlayablePlayer;
import com.pancik.libgdx.zumpatalentov.engine.ui.window.MainBuildingWindow;
import com.pancik.libgdx.zumpatalentov.entity.attackable.AttackableEntity;
import com.pancik.libgdx.zumpatalentov.entity.unit.FrankieUnit;
import com.pancik.libgdx.zumpatalentov.entity.unit.UnitEntity;
import com.pancik.libgdx.zumpatalentov.ui.ButtonHorizontalAlignment;
import com.pancik.libgdx.zumpatalentov.ui.ButtonVerticalAlignment;
import com.pancik.libgdx.zumpatalentov.util.MouseEvent;

public class MainBuilding extends BuildingEntity {
    public static Texture BASE_BUILDING_TEXTURE = DrawableData.textures.get("main_scientist");
    
    //protected static final int TEX_SIZE_X = 64;
    //protected static final int TEX_SIZE_Y = 64;
    //protected static final int[] ANIMATION_STEPS = {0,0,0,0,0,0,0,0, 1,1,1,1,1,1,1,1, 2,2,2,2,2,2,2,2, 3,3,3,3,3,3,3,3};
    
    protected static final int TEX_START_X = 0;
    protected static final int TEX_START_Y = 0;
    protected static final int TEX_SIZE_X = 256;
    protected static final int TEX_SIZE_Y = 256;
    protected static final int[] ANIMATION_STEPS = {0,0,0,0,0,0,0,0,
                                                    1,1,1,1,1,1,1,1,
                                                    2,2,2,2,2,2,2,2,
                                                    3,3,3,3,3,3,3,3,
                                                    4,4,4,4,4,4,4,4,
                                                    5,5,5,5,5,5,5,5,
                                                    6,6,6,6,6,6,6,6,
                                                    7,7,7,7,7,7,7,7};
    
    protected static final float DEFAULT_SIZE_X = 128.0f;
    protected static final float DEFAULT_SIZE_Y = 128.0f;
    
    protected static final int BUILDING_HEALTH = 1000;
    protected static final int SPAWN_COOLDOWN = 360;
    protected static final int INCOME_PER_MINUTE = 50;
    
    protected static final int REGENERATE_TIME = 30;
    protected static final int REGENERATE_AMOUNT = 1;
    
    protected Engine.EngineControls engineControls;
    protected boolean texureFlipped;
    protected int health = BUILDING_HEALTH;
    
    protected int spawnCooldownStep = 0;
    protected int spawnCooldownTime = SPAWN_COOLDOWN;
    
    protected int animationStep = 0;
    
    protected MainBuildingWindow window;
    
    protected int regenerateStep = 0;
    
    public MainBuilding(Vector2 pos, Vector2 size, int team, PlayablePlayer owner, Engine.EngineControls engineControls, boolean textureFlipped) {
        super(pos, size, team, owner);
        
        this.engineControls = engineControls;
        this.texureFlipped = textureFlipped;
        
        this.window = new MainBuildingWindow(ButtonHorizontalAlignment.MIDDLE, 0, ButtonVerticalAlignment.MIDDLE, 0, owner);
        this.window.hide();
        
        owner.getResourceManager().addIncomePerMinute(INCOME_PER_MINUTE);
    }
    
    @Override
    public void getHit(int damage, AttackableEntity from) {
        if(!isDead()) {
            health -= damage;

            PlayablePlayer playableOwner = (PlayablePlayer)owner;
            playableOwner.onMainBuildingHit(health, BUILDING_HEALTH);
            if(isDead()) {
                engineControls.addParticle(new BloodExplosionParticle(pos.add(-10.0f, -10.0f), size.x));
                engineControls.addParticle(new SmallExplosion(pos, size.x));
                engineControls.addParticle(new BloodExplosionParticle(pos.add(10.0f, 10.0f), size.x));
            }
        }
    }

    @Override
    public boolean isDead() {
        if(health <= 0) {
            return true;
        } else {
            return false;
        }
    }

    @Override
    public void tick() {
        if(!isDead()) {
            animationStep++;
            if(animationStep >= ANIMATION_STEPS.length) {
                animationStep = 0;
            }
            
            //spawnCooldownStep++;
            if(spawnCooldownStep >= spawnCooldownTime) {
                UnitEntity e = new FrankieUnit(owner.getSpawnPosition(), FrankieUnit.getDefaultSize(), team, owner, engineControls);
                e.setWaypointPath(owner.getRandomWaypointPath());
                engineControls.addEntity(e);
                //spawn
                spawnCooldownStep = 0;
            }
            
            Vector3 unprojectedPos = new Vector3(Gdx.input.getX(), Gdx.input.getY(), 0.0f);
            engineControls.getUICamera().unproject(unprojectedPos);
            window.tick(unprojectedPos.x, unprojectedPos.y);
            
            regenerateStep++;
            if(regenerateStep >= REGENERATE_TIME) {
                regenerateStep = 0;
                
                health += REGENERATE_AMOUNT;
                if(health > BUILDING_HEALTH) {
                    health = BUILDING_HEALTH;
                }
            }
        }
    }

    @Override
    public void render() {
        if(!isDead()) {
            DrawableData.spriteBatch.draw(BASE_BUILDING_TEXTURE,
                pos.x - size.x/2, pos.y - size.y/2, size.x, size.y,
                (TEX_SIZE_X * ANIMATION_STEPS[animationStep]) + TEX_START_X, TEX_START_Y, TEX_SIZE_X - TEX_START_X, TEX_SIZE_Y - TEX_START_Y, texureFlipped, true);
        }
    }

    @Override
    public boolean remove() {
        return isDead();
    }

    @Override
    public void onRemove() {
        owner.getResourceManager().addIncomePerMinute(-INCOME_PER_MINUTE);
    }

    @Override
    public float getPriority() {
        //return pos.y - 100.0f;
        return -1000.0f;
    }
    
    public static Vector2 getDefaultSize() {
        return new Vector2(DEFAULT_SIZE_X, DEFAULT_SIZE_Y);
    }

    protected boolean hitsBuilding(int x, int y) {
        if(x >= pos.x - size.x/2 && x <= pos.x + size.x/2 && y >= pos.y - size.y/2 && y <= pos.y + size.y/2) {
            return true;
        } else {
            return false;
        }
    }
    
    @Override
    public boolean handle(MouseEvent me) {
        boolean doHandle = hitsBuilding(me.getGameUnprojectedX(), me.getGameUnprojectedY()) && me.mouseDown() && !isDead();
        
        if(doHandle) {
            owner.setCurrentWindowAndShow(window);
        }
        
        return doHandle;
    }
}