package com.pancik.libgdx.zumpatalentov.engine.building;

import com.badlogic.gdx.graphics.Texture;
import com.badlogic.gdx.math.Vector2;
import com.pancik.libgdx.zumpatalentov.DrawableData;
import com.pancik.libgdx.zumpatalentov.engine.player.Player;
import com.pancik.libgdx.zumpatalentov.entity.attackable.AttackableEntity;
import com.pancik.libgdx.zumpatalentov.util.RenderUtils;

public class TofuTower extends TowerBuilding {
    public static Texture TOFU_TOWER_TEXTURE = DrawableData.textures.get("tofu_tower");
    
    public static final int TEX_SIZE_X = 256;
    public static final int TEX_SIZE_Y = 512;
    
    public static final int[] ANIMATION_STEPS = {0,
                                                 1,1,1,1,1,1,1,1,
                                                 2,2,2,2,2,2,2,2,
                                                 3,3,3,3,3,3,3,3,
                                                 4,4,4,4,4,4,4,4,
                                                 5,5,5,5,5,5,5,5};
    
    public static final int TOFU_TOWER_HEALTH = 300;
    public static final int TOFU_SPAWN_COOLDOWN = 300;
    
    public static final float DEFAULT_SIZE_X = 64.0f;
    public static final float DEFAULT_SIZE_Y = 64.0f;
    
    protected int health = TOFU_TOWER_HEALTH;
    
    protected int currentSpawnStep = 0;
    
    public TofuTower(Vector2 pos, Vector2 size, int team, Player owner) {
        super(pos, size, team, owner);
    }
    
    @Override
    public void towerTick() {
        if(currentSpawnStep >=  TOFU_SPAWN_COOLDOWN) {
            owner.getBonusManager().addReward(pos.cpy().sub(0.0f, size.y), pos.cpy().add(0.0f, size.y * 2));
            currentSpawnStep = 0;
        }
        currentSpawnStep++;
    }

    @Override
    public boolean attack(AttackableEntity e) {
        return false;
    }

    @Override
    public int getAttackCooldown() {
        return 0;
    }

    @Override
    public void getHit(int damage, AttackableEntity from) {
        if(!isDead()) {
            this.health -= damage;
        }
    }

    @Override
    public boolean isDead() {
        return (health <= 0);
    }

    @Override
    public void render() {
        if(!isDead()) {
            int id = 0;
            if(TOFU_SPAWN_COOLDOWN - currentSpawnStep < ANIMATION_STEPS.length) {
                id = (ANIMATION_STEPS.length - 1) - (TOFU_SPAWN_COOLDOWN - currentSpawnStep);
            }
            
            RenderUtils.blit(DrawableData.spriteBatch,
                    TOFU_TOWER_TEXTURE,
                    new RenderUtils.ScreenInfo(pos.x - size.x/2, pos.y - size.y/2 - size.y, size.x, size.y + size.y),
                    new RenderUtils.ScreenInfo(TEX_SIZE_X * ANIMATION_STEPS[id], 0, TEX_SIZE_X, TEX_SIZE_Y));
        }
    }

    @Override
    public boolean remove() {
        return isDead();
    }

    @Override
    public void onRemove() {
    }
    
    @Override
    public boolean hitsTexture(int posx, int posy) {
        if(posx >= pos.x - size.x/2 && posx <= pos.x + size.x/2 && posy >= pos.y - size.y/2 && posy <= pos.y + size.y/2) {
            return true;
        } else {
            return false;
        }
    }
    
    @Override
    public void forceRemove() {
        health = 0;
    }
    
    public static Vector2 getDefaultSize() {
        return new Vector2(DEFAULT_SIZE_X, DEFAULT_SIZE_Y);
    }
}
