package com.pancik.libgdx.zumpatalentov.engine.component;

import com.pancik.libgdx.zumpatalentov.engine.Level;
import com.pancik.libgdx.zumpatalentov.engine.tile.FloorTile;
import com.pancik.libgdx.zumpatalentov.engine.tile.GrassTile;
import com.pancik.libgdx.zumpatalentov.engine.tile.RoadTile;
import com.pancik.libgdx.zumpatalentov.engine.tile.Tile;
import com.pancik.libgdx.zumpatalentov.engine.tile.WallTile;

public class LevelTileManager extends TileManager {
    private int width;
    private int height; 
    
    private static final int GRASS_STRIP_SIZE = 20;
    
    public LevelTileManager(Level level) throws Exception {
        this.width = level.getWidth();
        this.height = level.getHeight();
        this.tiles = new Tile[level.getHeight() + GRASS_STRIP_SIZE * 2][level.getWidth() + GRASS_STRIP_SIZE * 2];
        
        for(int j = 0; j < tiles.length; j++) {
            for(int i = 0; i < tiles[j].length; i++) {
                tiles[j][i] = new GrassTile((i - GRASS_STRIP_SIZE) * Tile.TILE_SIZE, (j - GRASS_STRIP_SIZE) * Tile.TILE_SIZE);
            }
        }
        
        Tile.Type[][] layout = level.getTileLayout();
        
        for(int j = 0; j < level.getHeight(); j++) {
            for(int i = 0; i < level.getWidth(); i++) {
                if(layout[j][i] != null) {
                    int tempJ = GRASS_STRIP_SIZE  + j;
                    int tempI = GRASS_STRIP_SIZE  + i;
                    switch(layout[j][i]) {
                        case GRASS:
                            tiles[tempJ][tempI] = new GrassTile(i * Tile.TILE_SIZE, j * Tile.TILE_SIZE);
                            break;
                        case ROAD:
                            tiles[tempJ][tempI] = new RoadTile(i * Tile.TILE_SIZE, j * Tile.TILE_SIZE);
                            break;
                        case WALL:
                            tiles[tempJ][tempI] = new WallTile(i * Tile.TILE_SIZE, j * Tile.TILE_SIZE);
                            break;
                        case FLOOR:
                            tiles[tempJ][tempI] = new FloorTile(i * Tile.TILE_SIZE, j * Tile.TILE_SIZE);
                            break;
                        default:
                            throw new Exception("Missing tile handler for: " + layout[j][i]);
                    }
                }
            }
        }
        
        this.recalculateNeighbourTiles();
    }

    public int getMinX() {
        return 0;
    }
    
    public int getMaxX() {
        return width * Tile.TILE_SIZE;
    }
    
    public int getMinY() {
        return 0;
    }
    
    public int getMaxY() {
        return height * Tile.TILE_SIZE;
    }
}
