package com.pancik.libgdx.zumpatalentov.engine.component;

import com.pancik.libgdx.zumpatalentov.engine.tile.Tile;

public abstract class TileManager {

    protected Tile[][] tiles;
    
    public void tick() {
        if(tiles != null) {
            for(Tile[] row : tiles) {
                if(row != null) {
                    for(Tile t : row) {
                        if(t != null) {
                            t.tick();
                        }
                    }
                }
            }
        }
    }
    
    public void render() {
        if(tiles != null) {
            for(Tile[] row : tiles) {
                if(row != null) {
                    for(Tile t : row) {
                        if(t != null) {
                            t.render();
                        }
                    }
                }
            }
        }
    }
    
    public Tile[] getSurroundingTiles(float posx, float posy) {
        posx += (float)Tile.TILE_SIZE/2;
        posy += (float)Tile.TILE_SIZE/2;
        int locx = (int)Math.floor(posx/(float)Tile.TILE_SIZE);
        int locy = (int)Math.floor(posy/(float)Tile.TILE_SIZE);
        Tile[] out = new Tile[9];
        
        int[] dx = {0, -1, -1, 0, 1, 1, 1, 0, -1};
        int[] dy = {0, 0, 1, 1, 1, 0, -1, -1, -1};
        
        assert (dx.length == dy.length);
        
        for(int i = 0; i < dx.length; i++) {
            if(locy + dy[i] >= 0 && locy + dy[i] < tiles.length) {
                if(tiles[locy + dy[i]] != null) {
                    if(locx + dx[i] >= 0 && locx + dx[i] < tiles[locy + dy[i]].length) {
                        out[i] = tiles[locy + dy[i]][locx + dx[i]];
                    }
                }
            }
        }
        
        return out;
    }
    
    /*
    public boolean isWalkable(float posx, float posy) {
        posx += (float)Tile.TILE_SIZE/2;
        posy += (float)Tile.TILE_SIZE/2;
        int locx = (int)Math.floor(posx/(float)Tile.TILE_SIZE);
        int locy = (int)Math.floor(posy/(float)Tile.TILE_SIZE);
        
        if(tiles != null) {
            if(tiles.length > locy) {
                if(tiles[locy] != null) {
                    if(tiles[locy].length > locx) {
                        if(tiles[locy][locx] != null) {
                            return tiles[locy][locx].isWalkable();
                        }
                    }
                }
            }
        }
        return false;
    }
    */
    
    protected void recalculateNeighbourTiles() {
        int[] deltaIndexesX = { -1, +1, 0, 0 };
        int[] deltaIndexesY = { 0, 0, -1, +1 };
        int[] deltaIndexesBonus = { +4, +1, +8, +2};
        
        assert (deltaIndexesX.length == deltaIndexesY.length);
        
        if(tiles != null) {
            for(int k = 0; k < tiles.length; k++) {
                if(tiles[k] != null) {
                    for(int j = 0; j < tiles[k].length; j++) {
                        if(tiles[k][j] != null) {
                            int index = 0;
                            Tile neighbour = null;
                            for(int i = 0; i < deltaIndexesX.length; i++) {
                                int x = j + deltaIndexesX[i];
                                int y = k + deltaIndexesY[i];

                                if(x >= 0 && x < tiles[k].length && y >= 0 && y < tiles.length) {
                                    if(tiles[y][x] != null) {
                                        Tile current = tiles[k][j];
                                        Tile checking = tiles[y][x];

                                        if(current.getType() == checking.getType()) {
                                            index += deltaIndexesBonus[i];
                                        } else {
                                            neighbour = checking;
                                        }
                                    }
                                }
                            }

                            tiles[k][j].setTextureImageIndex(index);
                            tiles[k][j].setNeighbourTile(neighbour);
                        }
                    }
                }
            }
        }
    }
}
