package com.pancik.libgdx.zumpatalentov.engine.player;

import com.pancik.libgdx.zumpatalentov.engine.Engine.EngineControls;
import com.pancik.libgdx.zumpatalentov.engine.building.BaseBuildingSpot;
import com.pancik.libgdx.zumpatalentov.engine.building.BuildingSpotEntity;
import com.pancik.libgdx.zumpatalentov.engine.building.MutateLabBuilding;
import com.pancik.libgdx.zumpatalentov.engine.building.TowerBuildingSpot;
import com.pancik.libgdx.zumpatalentov.entity.unit.bodypart.HeadBodyPart;
import com.pancik.libgdx.zumpatalentov.entity.unit.bodypart.LegsBodyPart;
import com.pancik.libgdx.zumpatalentov.entity.unit.bodypart.TorsoBodyPart;
import com.pancik.libgdx.zumpatalentov.util.RandomGenerator;
import java.util.ArrayList;
import java.util.List;

public class AIPlayer extends PlayablePlayer {

    protected boolean savingForRescueStrike = false;
    
    protected boolean pickedBuilding = false;
    protected BuildingSpotEntity targetedSpot = null;
    protected BuildingSpotEntity.PossibilityCarrier targetPossibility = null;
    
    protected boolean pickedMutation = false;
    protected MutateLabBuilding mutationSpot = null;
    protected HeadBodyPart.Type head;
    protected TorsoBodyPart.Type torso;
    protected LegsBodyPart.Type legs;
    
    public AIPlayer(int number, int team, EngineControls engineControls) {
        super(number, team, engineControls);
    }

    protected int getTowerCount() {
        int count = 0;
        
        for(BuildingSpotEntity spot : buildingSpotEntities) {
            if(spot instanceof TowerBuildingSpot) {
                TowerBuildingSpot towerSpot = (TowerBuildingSpot)spot;
                if(!towerSpot.canBeBuiltOn()) {
                    count++;
                }
            }
        }
        
        return count;
    }
    
    protected int getBaseBuildingsCount() {
        int count = 0;
        
        for(BuildingSpotEntity spot : buildingSpotEntities) {
            if(spot instanceof BaseBuildingSpot) {
                BaseBuildingSpot baseSpot = (BaseBuildingSpot)spot;
                if(!baseSpot.canBeBuiltOn()) {
                    count++;
                }
            }
        }
        
        return count;
    }
    
    protected BuildingSpotEntity.PossibilityCarrier getRandomBuilding(BuildingSpotEntity spot) {
        if(spot instanceof TowerBuildingSpot) {
            switch(getTowerCount()) {
                case 0:
                    return TowerBuildingSpot.PossibleTowers.TESLA_COIL.getPossibilityCarrier();
            }
        } else if(spot instanceof BaseBuildingSpot) {
            switch(getBaseBuildingsCount()) {
                case 0:
                    return BaseBuildingSpot.PossibleBaseBuildings.FRANKIE_FACTORY.getPossibilityCarrier();
            }
        }
        
        BuildingSpotEntity.PossibilityCarrier[] possibleBuildings = targetedSpot.getBuildingPossibilities();
        return possibleBuildings[RandomGenerator.getRandom(possibleBuildings.length)];
    }
    
    @Override
    public void playerTick() {
        super.playerTick();

        bonusManager.forcePickupAll();
        
        //System.out.println("AI resources; coins: " + resourceManager.getCoinsAmount() + " tofu: " + resourceManager.getBonusCurrencyAmount());
        
        if (!savingForRescueStrike) {
            if (pickedBuilding == false) {
                List<Integer> possible = new ArrayList<Integer>();

                for (int i = 0; i < buildingSpotEntities.size(); i++) {
                    if (buildingSpotEntities.get(i).canBeBuiltOn()) {
                        possible.add(i);
                    }
                }

                if (!possible.isEmpty()) {
                    int index = possible.get(RandomGenerator.getRandom(possible.size()));
                    targetedSpot = buildingSpotEntities.get(index);


                    targetPossibility = getRandomBuilding(targetedSpot);

                    pickedBuilding = true;
                }
            } else {
                if (resourceManager.hasCurrencyFor(targetPossibility.cost)) {
                    targetedSpot.tryBuiltPossibility(targetPossibility.name);
                    if (!targetedSpot.canBeBuiltOn()) {
                        pickedBuilding = false;
                    }
                }
            }

            if (!pickedMutation) {
                for (BuildingSpotEntity spot : buildingSpotEntities) {
                    if (spot instanceof BaseBuildingSpot) {
                        BaseBuildingSpot base = (BaseBuildingSpot) spot;

                        if (base.getBuilding() != null) {
                            if (base.getBuilding() instanceof MutateLabBuilding) {
                                MutateLabBuilding building = (MutateLabBuilding) base.getBuilding();

                                if (!building.isProducing()) {
                                    head = HeadBodyPart.Type.values()[RandomGenerator.getRandom(HeadBodyPart.Type.values().length)];
                                    torso = TorsoBodyPart.Type.values()[RandomGenerator.getRandom(TorsoBodyPart.Type.values().length)];
                                    legs = LegsBodyPart.Type.values()[RandomGenerator.getRandom(LegsBodyPart.Type.values().length)];

                                    mutationSpot = building;
                                    pickedMutation = true;
                                }
                            }
                        }
                    }
                }
            } else {
                if (mutationSpot.remove()) {
                    mutationSpot = null;
                    pickedMutation = false;
                } else {
                    if (mutationSpot.chooseBodyParts(head, torso, legs)) {
                        pickedMutation = false;
                    }
                }
            }
        } else {
            if(resourceManager.hasCurrencyFor(RESCUE_STRIKE_COST)) {
                if(doRescueStrike()) {
                    savingForRescueStrike = false;
                }
            }
        }
    }

    @Override
    public void onMainBuildingHit(int remainingHealth, int maxHealth) {
        if((float)remainingHealth/(float)maxHealth < 0.90f) {
            savingForRescueStrike = true;
        }
    }

    @Override
    public void onBaseBuildingHit(int remainingHealth, int maxHealth) {
        if((float)remainingHealth/(float)maxHealth < 0.50f) {
            savingForRescueStrike = true;
        }
    }
    
    @Override
    public boolean isLocal() {
        return false;
    }
}
