package com.pancik.libgdx.zumpatalentov.engine.player.manager;

import com.badlogic.gdx.math.Vector2;
import com.pancik.libgdx.zumpatalentov.engine.Engine;
import com.pancik.libgdx.zumpatalentov.engine.ui.BonusCurrencyItem;
import com.pancik.libgdx.zumpatalentov.engine.ui.Icon;
import com.pancik.libgdx.zumpatalentov.util.MouseEvent;
import java.util.ArrayList;
import java.util.List;

public class BonusManager {
    protected ResourceManager resourceManager;
    protected Engine.EngineControls engineControls;

    protected List<BonusCurrencyItem> items = new ArrayList<BonusCurrencyItem>();
    
    public BonusManager(ResourceManager resourceManager, Engine.EngineControls engineControls) {
        this.resourceManager = resourceManager;
        this.engineControls = engineControls;
    }
    
    public void tick() {
        for(BonusCurrencyItem item : items) {
            item.tick();
        }
        
        for(BonusCurrencyItem item : items) {
            if(item.remove()) {
                items.remove(item);
                break;
            }
        }
    }
    
    public void renderUI() {
        for(BonusCurrencyItem item : items) {
            item.render();
        }
    }
    
    public boolean handle(MouseEvent me) {
        boolean out = false;
        for(BonusCurrencyItem item : items) {
            if(item.handleUIMouse(me)) {
                out = true;
            }
        }
        return out;
    }
    
    protected void addBonusItem(BonusCurrencyItem item) {
        items.add(item);
    }
    
    public void forcePickupAll() {
        for(BonusCurrencyItem item : items) {
            item.forcePickup();
        }
    }
    
    public void addReward(Vector2 gamePos, Vector2 groundPos) {
        BonusCurrencyItem item = new BonusCurrencyItem(Icon.TOFU, gamePos, groundPos, engineControls, new BonusCurrencyItem.CallbackHandler() {
            @Override
            public void onCurrencyAdded() {
                resourceManager.addBonusCurrency(10);
            }
        });
        
        addBonusItem(item);
    }
}
