package com.pancik.libgdx.zumpatalentov.engine.ui;

import com.badlogic.gdx.math.Vector2;
import com.pancik.libgdx.zumpatalentov.DrawableData;
import com.pancik.libgdx.zumpatalentov.engine.player.Player;
import com.pancik.libgdx.zumpatalentov.util.MouseEvent;

public class IconButtonClickable {
    public static interface CallbackHandler {
        public void onClicked();

        public boolean showTooltip();
        public Tooltip getTooltip(float posx, float posy);
        public Player getTooltipPlayer();
    }
    
    protected static int ICON_SIZE_X = 48;
    protected static int ICON_SIZE_Y = 48;
    
    protected Icon icon;
    protected Vector2 pos;
    protected CallbackHandler callback;
    
    protected Vector2 mousePos;

    public IconButtonClickable(Icon icon, Vector2 pos, CallbackHandler callback) {
        this.icon = icon;
        this.pos = pos;
        this.callback = callback;
    }

    public void tick(float mousePosX, float mousePosY) {
        if(callback.showTooltip()) {
            if(hitsIcon((int)mousePosX, (int)mousePosY)) {
                callback.getTooltipPlayer().setTooltip(callback.getTooltip(mousePosX + 10, mousePosY + 10));
            }
        }
    }
    
    public void render() {
        boolean hits = false;
        if(mousePos != null) {
            if(hitsIcon((int)mousePos.x, (int)mousePos.y)) {
                hits = true;
                DrawableData.spriteBatch.setColor(0.5f, 0.5f, 0.5f, 1.0f);
            }
        }
        
        icon.render((int)pos.x, (int)pos.y, ICON_SIZE_X, ICON_SIZE_Y);

        if(hits) {
            DrawableData.spriteBatch.setColor(1.0f, 1.0f, 1.0f, 1.0f);
        }
    }
    
    public boolean hitsIcon(int posx, int posy) {
        if(posx >= pos.x - ICON_SIZE_X/2 && posx <= pos.x + ICON_SIZE_X/2 && posy >= pos.y - ICON_SIZE_Y/2 && posy <= pos.y + ICON_SIZE_Y/2) {
            return true;
        } else {
            return false;
        }
    }
    
    public boolean handleUIMouse(MouseEvent me) {
        if(me.mouseDown()) {
            mousePos = new Vector2(me.getUiUnprojectedX(), me.getUiUnprojectedY());
        } else if(me.mouseDragged()) {
            if(mousePos != null) {
                mousePos.set(me.getUiUnprojectedX(), me.getUiUnprojectedY());
            }
        } else if(me.mouseUp()) {
            if(mousePos != null) {
                mousePos = null;
                
                if(hitsIcon(me.getUiUnprojectedX(), me.getUiUnprojectedY())) {
                    callback.onClicked();
                    return false;
                }
            }
        }
        return true;
    }

    public void resetInput() {
        mousePos = null;
    }
    
    public void setPos(Vector2 pos) {
        this.pos = pos;
    }
}
