package com.pancik.libgdx.zumpatalentov.engine.ui.window;

import com.pancik.libgdx.zumpatalentov.SoundData;
import com.pancik.libgdx.zumpatalentov.engine.player.PlayablePlayer;
import com.pancik.libgdx.zumpatalentov.engine.player.manager.ResourceManager;
import com.pancik.libgdx.zumpatalentov.engine.player.manager.ResourceManager.CostWrapper;
import com.pancik.libgdx.zumpatalentov.engine.ui.Icon;
import com.pancik.libgdx.zumpatalentov.engine.ui.Tooltip;
import com.pancik.libgdx.zumpatalentov.entity.unit.buff.upgrade.PermanentAttackPowerUpgrade;
import com.pancik.libgdx.zumpatalentov.entity.unit.buff.upgrade.PermanentAttackSpeedUpgrade;
import com.pancik.libgdx.zumpatalentov.entity.unit.buff.upgrade.PermanentHealthUpgrade;
import com.pancik.libgdx.zumpatalentov.entity.unit.buff.upgrade.PermanentSpawnSpeedUpgrade;
import com.pancik.libgdx.zumpatalentov.entity.unit.buff.upgrade.PermanentSpeedUpgrade;
import com.pancik.libgdx.zumpatalentov.entity.unit.buff.upgrade.TemporaryAttackPowerUpgrade;
import com.pancik.libgdx.zumpatalentov.entity.unit.buff.upgrade.TemporaryAttackSpeedUpgrade;
import com.pancik.libgdx.zumpatalentov.entity.unit.buff.upgrade.TemporarySpawnSpeedUpgrade;
import com.pancik.libgdx.zumpatalentov.entity.unit.buff.upgrade.TemporarySpeedUpgrade;
import com.pancik.libgdx.zumpatalentov.ui.ButtonHorizontalAlignment;
import com.pancik.libgdx.zumpatalentov.ui.ButtonImageEnum;
import com.pancik.libgdx.zumpatalentov.ui.ButtonVerticalAlignment;
import com.pancik.libgdx.zumpatalentov.util.RenderUtils;

public class MainBuildingWindow extends Window {
    public static enum Upgrades {
        TEMPORARY_MOVEMENT_SPEED (new CostWrapper(30, 30), TemporarySpeedUpgrade.LIFETIME),
        TEMPORARY_ATTACK_POWER (new CostWrapper(30, 30), TemporaryAttackPowerUpgrade.LIFETIME), 
        TEMPORARY_ATTACK_SPEED (new CostWrapper(30, 30), TemporaryAttackSpeedUpgrade.LIFETIME),
        TEMPORARY_SPAWN_SPEED (new CostWrapper(30, 30), TemporarySpawnSpeedUpgrade.LIFETIME),
        PERMANENT_MOVEMENT_SPEED (new CostWrapper(50, 50), 0),
        PERMANENT_ATTACK_POWER (new CostWrapper(50, 50), 0),
        PERMANENT_ATTACK_SPEED (new CostWrapper(50, 50), 0),
        PERMANENT_SPAWN_SPEED (new CostWrapper(50, 50), 0),
        PERMANENT_HEALTH_INCREASE (new CostWrapper(50, 50), 0);
        
        private ResourceManager.CostWrapper cost;
        private int cooldown;

        private Upgrades(CostWrapper cost, int cooldown) {
            this.cost = cost;
            this.cooldown = cooldown;
        }

        public int getCooldown() {
            return cooldown;
        }

        public CostWrapper getCost() {
            return cost;
        }
    }
    
    protected PlayablePlayer owner;
    
    public MainBuildingWindow(ButtonHorizontalAlignment horizontal, int deltaX, ButtonVerticalAlignment vertical, int deltaY, PlayablePlayer owner) {
        super(horizontal, deltaX, vertical, deltaY, 384, 314);
        this.owner = owner;
        
        int heightTop = 74;
        int heightBottom = heightTop + 64 + 20 + 30;
        
        Tooltip temporaryMovementSpeed = new Tooltip(Upgrades.TEMPORARY_MOVEMENT_SPEED.getCost(), "Movement speed", new String[] {
            "Increases movement speed by 100% for 20 seconds."
        });
        
        Tooltip temporaryAttackPower = new Tooltip(Upgrades.TEMPORARY_ATTACK_POWER.getCost(), "Attack power", new String[] {
            "Increases attack damage by 50% for 20 seconds."
        });
        
        Tooltip temporaryAttackSpeed = new Tooltip(Upgrades.TEMPORARY_ATTACK_SPEED.getCost(), "Attack speed", new String[] {
            "Increases attack speed by 50% for 20 seconds."
        });
        
        Tooltip temporarySpawnSpeed = new Tooltip(Upgrades.TEMPORARY_SPAWN_SPEED.getCost(), "Spawn speed", new String[] {
            "Decreases spawn speed by 30% for 20 seconds."
        });
        
        Tooltip permanentMovementSpeed = new Tooltip(Upgrades.PERMANENT_MOVEMENT_SPEED.getCost(), "Permanent movement speed", new String[] {
            "Increases movement speed by 10%."
        });
        
        Tooltip permanentAttackPower = new Tooltip(Upgrades.PERMANENT_ATTACK_POWER.getCost(), "Permanent attack power", new String[] {
            "Increases attack power by 10%."
        });
        
        Tooltip permanentAttackSpeed = new Tooltip(Upgrades.PERMANENT_ATTACK_SPEED.getCost(), "Permanent attack speed", new String[] {
            "Increases attack speed by 10%."
        });
        
        Tooltip permanentSpawnSpeed = new Tooltip(Upgrades.PERMANENT_SPAWN_SPEED.getCost(), "Permanent spawn speed", new String[] {
            "Decreases spawn time by 10%."
        });
        
        Tooltip permanentHealthIncrease = new Tooltip(Upgrades.PERMANENT_HEALTH_INCREASE.getCost(), "Permanent health increase", new String[] {
            "Increases health of all units by 10%."
        });
        
        Tooltip rescueStrike = new Tooltip(PlayablePlayer.RESCUE_STRIKE_COST, "Rescue Strike", new String[] {
            "Instantly kills all enemy units surrounding scientist.", "Has 20 seconds cooldown."
        });
        
        
        this.uiElements.add(new WindowText(ButtonHorizontalAlignment.MIDDLE,
                0,
                ButtonVerticalAlignment.TOP,
                44,
                "TEMPORARY UPGRADES",
                RenderUtils.TextAlligment.MIDDLE, RenderUtils.TextAlligment.BOTTOM));
        
        this.uiElements.add(new WindowCooldownButton(Icon.UPGRADE_MOV_SPEED,
                ButtonHorizontalAlignment.LEFT,
                12,
                ButtonVerticalAlignment.TOP,
                heightTop,
                "",
                ButtonImageEnum.WINDOW_RECTANGLE_BUTTON_64_64,
                new WindowCooldownButton.Callback() {

                    @Override
                    public boolean onButtonPressed() {
                        return applyUpgrade(Upgrades.TEMPORARY_MOVEMENT_SPEED);
                    }
                },
                calculateX(), calculateY(),
                sizeX, sizeY,
                Upgrades.TEMPORARY_MOVEMENT_SPEED.getCooldown(), temporaryMovementSpeed, owner));
        
        this.uiElements.add(new WindowCooldownButton(Icon.UPGRADE_MUSCLE,
                ButtonHorizontalAlignment.LEFT,
                86,
                ButtonVerticalAlignment.TOP,
                heightTop,
                "",
                ButtonImageEnum.WINDOW_RECTANGLE_BUTTON_64_64,
                new WindowCooldownButton.Callback() {

                    @Override
                    public boolean onButtonPressed() {
                        return applyUpgrade(Upgrades.TEMPORARY_ATTACK_POWER);
                    }
                },
                calculateX(),
                calculateY(),
                sizeX,
                sizeY,
                Upgrades.TEMPORARY_ATTACK_POWER.getCooldown(), temporaryAttackPower, owner));
        
        this.uiElements.add(new WindowCooldownButton(Icon.UPGRADE_SPAWN_SPEED,
                ButtonHorizontalAlignment.LEFT,
                160,
                ButtonVerticalAlignment.TOP,
                heightTop,
                "",
                ButtonImageEnum.WINDOW_RECTANGLE_BUTTON_64_64,
                new WindowCooldownButton.Callback() {

                    @Override
                    public boolean onButtonPressed() {
                        return applyUpgrade(Upgrades.TEMPORARY_SPAWN_SPEED);
                    }
                },
                calculateX(),
                calculateY(),
                sizeX,
                sizeY,
                Upgrades.TEMPORARY_SPAWN_SPEED.getCooldown(), temporarySpawnSpeed, owner));
        
        this.uiElements.add(new WindowCooldownButton(Icon.UPGRADE_ATTACK_SPEED,
                ButtonHorizontalAlignment.LEFT,
                234,
                ButtonVerticalAlignment.TOP,
                heightTop,
                "",
                ButtonImageEnum.WINDOW_RECTANGLE_BUTTON_64_64,
                new WindowCooldownButton.Callback() {

                    @Override
                    public boolean onButtonPressed() {
                        return applyUpgrade(Upgrades.TEMPORARY_ATTACK_SPEED);
                    }
                },
                calculateX(),
                calculateY(),
                sizeX,
                sizeY,
                Upgrades.TEMPORARY_ATTACK_SPEED.getCooldown(), temporaryAttackSpeed, owner));
        
        this.uiElements.add(new WindowCooldownButton(Icon.UPGRADE_EXPLOSION,
                ButtonHorizontalAlignment.LEFT,
                308,
                ButtonVerticalAlignment.TOP,
                heightTop,
                "",
                ButtonImageEnum.WINDOW_RECTANGLE_BUTTON_64_64,
                new WindowCooldownButton.Callback() {

                    @Override
                    public boolean onButtonPressed() {
                        boolean result = MainBuildingWindow.this.owner.doRescueStrike();
                        if(result) {
                            hide();
                        }
                        return result;
                    }
                },
                calculateX(),
                calculateY(),
                sizeX,
                sizeY,
                PlayablePlayer.RESCUE_STRIKE_COOLDOWN, rescueStrike, owner));
        
        this.uiElements.add(new WindowText(ButtonHorizontalAlignment.MIDDLE,
                0,
                ButtonVerticalAlignment.TOP,
                heightTop + 64 + 20,
                "PERMANENT UPGRADES",
                RenderUtils.TextAlligment.MIDDLE, RenderUtils.TextAlligment.BOTTOM));
        
        this.uiElements.add(new WindowCooldownButton(Icon.UPGRADE_PERMANENT_MOV_SPEED,
                ButtonHorizontalAlignment.LEFT,
                12,
                ButtonVerticalAlignment.TOP,
                heightBottom,
                "",
                ButtonImageEnum.WINDOW_RECTANGLE_BUTTON_64_64,
                new WindowCooldownButton.Callback() {

                    @Override
                    public boolean onButtonPressed() {
                        return applyUpgrade(Upgrades.PERMANENT_MOVEMENT_SPEED);
                    }
                },
                calculateX(),
                calculateY(),
                sizeX,
                sizeY,
                0, permanentMovementSpeed, owner));
        
        this.uiElements.add(new WindowCooldownButton(Icon.UPGRADE_PERMANENT_MUSCLE,
                ButtonHorizontalAlignment.LEFT,
                86,
                ButtonVerticalAlignment.TOP,
                heightBottom,
                "",
                ButtonImageEnum.WINDOW_RECTANGLE_BUTTON_64_64,
                new WindowCooldownButton.Callback() {

                    @Override
                    public boolean onButtonPressed() {
                        return applyUpgrade(Upgrades.PERMANENT_ATTACK_POWER);
                    }
                },
                calculateX(),
                calculateY(),
                sizeX,
                sizeY,
                0, permanentAttackPower, owner));
        
        this.uiElements.add(new WindowCooldownButton(Icon.UPGRADE_PERMANENT_SPAWN_SPEED,
                ButtonHorizontalAlignment.LEFT,
                160,
                ButtonVerticalAlignment.TOP,
                heightBottom,
                "",
                ButtonImageEnum.WINDOW_RECTANGLE_BUTTON_64_64,
                new WindowCooldownButton.Callback() {

                    @Override
                    public boolean onButtonPressed() {
                        return applyUpgrade(Upgrades.PERMANENT_SPAWN_SPEED);
                    }
                },
                calculateX(),
                calculateY(),
                sizeX,
                sizeY,
                0, permanentSpawnSpeed, owner));
        
        this.uiElements.add(new WindowCooldownButton(Icon.UPGRADE_PERMANENT_ATTACK_SPEED,
                ButtonHorizontalAlignment.LEFT,
                234,
                ButtonVerticalAlignment.TOP,
                heightBottom,
                "",
                ButtonImageEnum.WINDOW_RECTANGLE_BUTTON_64_64,
                new WindowCooldownButton.Callback() {

                    @Override
                    public boolean onButtonPressed() {
                        return applyUpgrade(Upgrades.PERMANENT_ATTACK_SPEED);
                    }
                },
                calculateX(),
                calculateY(),
                sizeX,
                sizeY,
                0, permanentAttackSpeed, owner));
        
        this.uiElements.add(new WindowCooldownButton(Icon.UPGRADE_HEART,
                ButtonHorizontalAlignment.LEFT,
                308,
                ButtonVerticalAlignment.TOP,
                heightBottom,
                "",
                ButtonImageEnum.WINDOW_RECTANGLE_BUTTON_64_64,
                new WindowCooldownButton.Callback() {

                    @Override
                    public boolean onButtonPressed() {
                        return applyUpgrade(Upgrades.PERMANENT_HEALTH_INCREASE);
                    }
                },
                calculateX(),
                calculateY(),
                sizeX,
                sizeY,
                0, permanentHealthIncrease, owner));
        
        this.uiElements.add(new WindowText(ButtonHorizontalAlignment.MIDDLE,
                0,
                ButtonVerticalAlignment.TOP,
                heightBottom + 64 + 20,
                "Press the upgrade to buy it!",
                RenderUtils.TextAlligment.MIDDLE, RenderUtils.TextAlligment.BOTTOM));
    }

    public boolean applyUpgrade(Upgrades upgrade) {
        if(owner.getResourceManager().hasCurrencyFor(upgrade.cost)) {
            owner.getResourceManager().removeCurrency(upgrade.cost);
            
            if(owner.isLocal()) {
                SoundData.playRandomSound("coins_1", "coins_2");
            }
            
            switch(upgrade) {
                case PERMANENT_ATTACK_POWER:
                    owner.addUpgrade(new PermanentAttackPowerUpgrade());
                    break;
                case PERMANENT_ATTACK_SPEED:
                    owner.addUpgrade(new PermanentAttackSpeedUpgrade());
                    break;
                case PERMANENT_HEALTH_INCREASE:
                    owner.addUpgrade(new PermanentHealthUpgrade());
                    break;
                case PERMANENT_MOVEMENT_SPEED:
                    owner.addUpgrade(new PermanentSpeedUpgrade());
                    break;
                case PERMANENT_SPAWN_SPEED:
                    owner.addUpgrade(new PermanentSpawnSpeedUpgrade());
                    break;
                case TEMPORARY_ATTACK_POWER:
                    owner.addUpgrade(new TemporaryAttackPowerUpgrade());
                    break;
                case TEMPORARY_ATTACK_SPEED:
                    owner.addUpgrade(new TemporaryAttackSpeedUpgrade());
                    break;
                case TEMPORARY_MOVEMENT_SPEED:
                    owner.addUpgrade(new TemporarySpeedUpgrade());
                    break;
                case TEMPORARY_SPAWN_SPEED:
                    owner.addUpgrade(new TemporarySpawnSpeedUpgrade());
                    break;
            }
            
            return true;
        } else {
            owner.addErrorMessage("Insufficient funds!");
            return false;
        }
    }

    @Override
    protected void windowTick(float mousePosX, float mousePosY) {
    }

    @Override
    protected void windowRender(int windowX, int windowY, int sizeX, int sizeY) {
    }
}
