package com.pancik.libgdx.zumpatalentov.engine.ui.window;

import com.badlogic.gdx.graphics.Color;
import com.badlogic.gdx.graphics.g2d.BitmapFont;
import com.pancik.libgdx.zumpatalentov.DrawableData;
import com.pancik.libgdx.zumpatalentov.engine.player.Player;
import com.pancik.libgdx.zumpatalentov.engine.player.manager.ResourceManager.CostWrapper;
import com.pancik.libgdx.zumpatalentov.engine.ui.Icon;
import com.pancik.libgdx.zumpatalentov.engine.ui.Tooltip;
import com.pancik.libgdx.zumpatalentov.entity.unit.bodypart.HeadBodyPart;
import com.pancik.libgdx.zumpatalentov.entity.unit.bodypart.LegsBodyPart;
import com.pancik.libgdx.zumpatalentov.entity.unit.bodypart.TorsoBodyPart;
import com.pancik.libgdx.zumpatalentov.ui.ButtonHorizontalAlignment;
import com.pancik.libgdx.zumpatalentov.ui.ButtonImageEnum;
import com.pancik.libgdx.zumpatalentov.ui.ButtonVerticalAlignment;
import com.pancik.libgdx.zumpatalentov.ui.ClickableButton;
import com.pancik.libgdx.zumpatalentov.util.RenderUtils;

public class MutateLabWindow extends Window {
    public static interface Callback {
        public void onBuildingDestroy();
        public boolean onPartsChosen(HeadBodyPart.Type head, TorsoBodyPart.Type torso, LegsBodyPart.Type legs);
        public Player getTooltipPlayer();
    }
    
    private static class BodyPartContainer {
        public Icon icon;
        public String text;
        public Tooltip tooltip;

        public BodyPartContainer(Icon icon, String text, Tooltip tooltip) {
            this.icon = icon;
            this.text = text;
            this.tooltip = tooltip;
        }
    }

    protected Callback callback;
    
    private BodyPartContainer[] head;
    private BodyPartContainer[] torso;
    private BodyPartContainer[] legs;
    
    private int currentHead = 0;
    private int currentTorso = 0;
    private int currentLegs = 0;
    
    private WindowWellIcon wellHead;
    private WindowWellIcon wellTorso;
    private WindowWellIcon wellLegs;
    
    private WindowText totalCost;
    
    public MutateLabWindow(ButtonHorizontalAlignment horizontal, int deltaX, ButtonVerticalAlignment vertical, int deltaY, Callback callback) {
        super(horizontal, deltaX, vertical, deltaY, 384, 384);
        this.callback = callback;
        
        head = new BodyPartContainer[] {
            new BodyPartContainer(Icon.BODY_PART_HEAD_BATMAN, HeadBodyPart.Type.BATMAN.name(),
                new Tooltip(HeadBodyPart.Type.BATMAN.getCost().coinsAmount, HeadBodyPart.Type.BATMAN.getCost().bonusAmount,
                    "Wooden Head", new String[] {"Has lowest health points."})),
            
            new BodyPartContainer(Icon.BODY_PART_HEAD_BIGNOSE, HeadBodyPart.Type.BIGNOSE.name(),
                new Tooltip(HeadBodyPart.Type.BIGNOSE.getCost().coinsAmount, HeadBodyPart.Type.BIGNOSE.getCost().bonusAmount,
                    "Solid Head", new String[] {"Medium health points."})),
            
            new BodyPartContainer(Icon.BODY_PART_HEAD_BUNNY, HeadBodyPart.Type.BUNNY.name(),
                new Tooltip(HeadBodyPart.Type.BUNNY.getCost().coinsAmount, HeadBodyPart.Type.BUNNY.getCost().bonusAmount,
                    "Bunny Head", new String[] {"The biggest amount of health points."})),
        };
        
        torso = new BodyPartContainer[] {
            new BodyPartContainer(Icon.BODY_PART_TORSO_MELEE, TorsoBodyPart.Type.MELEE.name(),
                new Tooltip(TorsoBodyPart.Type.MELEE.getCost().coinsAmount, TorsoBodyPart.Type.MELEE.getCost().bonusAmount,
                    "Melee Torso", new String[] {"Even when this torso has to come to the enemy.", "It certainly doesn't lack brutality."})),
            new BodyPartContainer(Icon.BODY_PART_TORSO_ONEHAND, TorsoBodyPart.Type.ONEHAND.name(),
                new Tooltip(TorsoBodyPart.Type.ONEHAND.getCost().coinsAmount, TorsoBodyPart.Type.ONEHAND.getCost().bonusAmount,
                    "Torso With Pistol", new String[] {"Guns. Guns. Guns. Laser pistol is great for short range."})),
            new BodyPartContainer(Icon.BODY_PART_TORSO_TWOHAND, TorsoBodyPart.Type.TWOHAND.name(),
                new Tooltip(TorsoBodyPart.Type.TWOHAND.getCost().coinsAmount, TorsoBodyPart.Type.TWOHAND.getCost().bonusAmount,
                    "Torso With Rifle", new String[] {"Laser rifle? You definitely want this gun!"})),
        };
        
        legs = new BodyPartContainer[] {
            new BodyPartContainer(Icon.BODY_PART_LEGS_TWO, LegsBodyPart.Type.TWO.name(),
                new Tooltip(LegsBodyPart.Type.TWO.getCost().coinsAmount, LegsBodyPart.Type.TWO.getCost().bonusAmount, 
                    "Two Legs", new String[] {"Like those you have.", "Not very good."})),
            
            new BodyPartContainer(Icon.BODY_PART_LEGS_THREE, LegsBodyPart.Type.THREE.name(),
                new Tooltip(LegsBodyPart.Type.THREE.getCost().coinsAmount, LegsBodyPart.Type.THREE.getCost().bonusAmount,
                    "Three Legs", new String[] {"A little bit faster, better for running."})),
            
            new BodyPartContainer(Icon.BODY_PART_LEGS_FOUR, LegsBodyPart.Type.FOUR.name(),
                new Tooltip(LegsBodyPart.Type.FOUR.getCost().coinsAmount, LegsBodyPart.Type.FOUR.getCost().bonusAmount,
                    "Four Legs", new String[] {"The fastest and the fanciest one.", "Very good for charging the enemy head on!"})),
        };
        
        //int posWellX = 94;
        int posWellY[] = {44, 122, 206};

        this.uiElements.add(new WindowText(ButtonHorizontalAlignment.MIDDLE,
                0,
                ButtonVerticalAlignment.TOP,
                8,
                "Mutate a friend!",
                RenderUtils.TextAlligment.MIDDLE,
                RenderUtils.TextAlligment.BOTTOM,
                Color.RED));
        
        {    
            this.uiElements.add(new WindowButton(ButtonHorizontalAlignment.MIDDLE,
                    -74,
                    ButtonVerticalAlignment.TOP,
                    posWellY[0],
                    "",
                    ButtonImageEnum.WINDOW_ARROW_LEFT_64_64,
                    new ClickableButton.CallbackHandler() {

                        @Override
                        public void onButtonPressed() {
                            currentHead = changeIndex(head.length, currentHead, -1);
                            wellHead.set(head[currentHead].icon, head[currentHead].tooltip);
                        }
                    },
                    calculateX(),
                    calculateY(),
                    sizeX,
                    sizeY));

            wellHead = new WindowWellIcon(head[currentHead].icon,
                    head[currentHead].tooltip,
                    callback.getTooltipPlayer(),
                    ButtonHorizontalAlignment.MIDDLE,
                    0,
                    ButtonVerticalAlignment.TOP,
                    posWellY[0],
                    64,
                    64);
            this.uiElements.add(wellHead);
            
            this.uiElements.add(new WindowButton(ButtonHorizontalAlignment.MIDDLE,
                    74,
                    ButtonVerticalAlignment.TOP,
                    posWellY[0],
                    "",
                    ButtonImageEnum.WINDOW_ARROW_RIGHT_64_64,
                    new ClickableButton.CallbackHandler() {

                        @Override
                        public void onButtonPressed() {
                            currentHead = changeIndex(head.length, currentHead, 1);
                            wellHead.set(head[currentHead].icon, head[currentHead].tooltip);
                        }
                    },
                    calculateX(),
                    calculateY(),
                    sizeX,
                    sizeY));
        }
        
        {
            this.uiElements.add(new WindowButton(ButtonHorizontalAlignment.MIDDLE,
                    -74,
                    ButtonVerticalAlignment.TOP,
                    posWellY[1],
                    "",
                    ButtonImageEnum.WINDOW_ARROW_LEFT_64_64,
                    new ClickableButton.CallbackHandler() {

                        @Override
                        public void onButtonPressed() {
                            currentTorso = changeIndex(torso.length, currentTorso, -1);
                            wellTorso.set(torso[currentTorso].icon, torso[currentTorso].tooltip);
                        }
                    },
                    calculateX(),
                    calculateY(),
                    sizeX,
                    sizeY));
            
            wellTorso = new WindowWellIcon(torso[currentTorso].icon,
                    torso[currentTorso].tooltip,
                    callback.getTooltipPlayer(),
                    ButtonHorizontalAlignment.MIDDLE,
                    0,
                    ButtonVerticalAlignment.TOP,
                    posWellY[1],
                    64,
                    64);
            this.uiElements.add(wellTorso);
            
            this.uiElements.add(new WindowButton(ButtonHorizontalAlignment.MIDDLE,
                    74,
                    ButtonVerticalAlignment.TOP,
                    posWellY[1],
                    "",
                    ButtonImageEnum.WINDOW_ARROW_RIGHT_64_64,
                    new ClickableButton.CallbackHandler() {

                        @Override
                        public void onButtonPressed() {
                            currentTorso = changeIndex(torso.length, currentTorso, 1);
                            wellTorso.set(torso[currentTorso].icon, torso[currentTorso].tooltip);
                        }
                    },
                    calculateX(),
                    calculateY(),
                    sizeX,
                    sizeY));
        }
        
        {
            this.uiElements.add(new WindowButton(ButtonHorizontalAlignment.MIDDLE,
                    -74,
                    ButtonVerticalAlignment.TOP,
                    posWellY[2],
                    "",
                    ButtonImageEnum.WINDOW_ARROW_LEFT_64_64,
                    new ClickableButton.CallbackHandler() {

                        @Override
                        public void onButtonPressed() {
                            currentLegs = changeIndex(legs.length, currentLegs, -1);
                            wellLegs.set(legs[currentLegs].icon, legs[currentLegs].tooltip);
                        }
                    },
                    calculateX(),
                    calculateY(),
                    sizeX,
                    sizeY));
            
            wellLegs = new WindowWellIcon(legs[currentLegs].icon,
                    legs[currentLegs].tooltip,
                    callback.getTooltipPlayer(),
                    ButtonHorizontalAlignment.MIDDLE,
                    0,
                    ButtonVerticalAlignment.TOP,
                    posWellY[2],
                    64,
                    64);
            this.uiElements.add(wellLegs);
            
            this.uiElements.add(new WindowButton(ButtonHorizontalAlignment.MIDDLE,
                    74,
                    ButtonVerticalAlignment.TOP,
                    posWellY[2],
                    "",
                    ButtonImageEnum.WINDOW_ARROW_RIGHT_64_64,
                    new ClickableButton.CallbackHandler() {

                        @Override
                        public void onButtonPressed() {
                            currentLegs = changeIndex(legs.length, currentLegs, 1);
                            wellLegs.set(legs[currentLegs].icon, legs[currentLegs].tooltip);
                        }
                    },
                    calculateX(),
                    calculateY(),
                    sizeX,
                    sizeY));
        }
        
        totalCost = new WindowText(ButtonHorizontalAlignment.MIDDLE,
                0,
                ButtonVerticalAlignment.BOTTOM,
                -79,
                "HELPER FOR X/Y COORDS, DO NOT ADD TO UI ELEMENTS!",
                RenderUtils.TextAlligment.MIDDLE, RenderUtils.TextAlligment.TOP);
        
        this.uiElements.add(new WindowButton(ButtonHorizontalAlignment.MIDDLE,
                    0,
                    ButtonVerticalAlignment.BOTTOM,
                    -5,
                    "MUTATE!",
                    ButtonImageEnum.WINDOW_BIG_BUTTON_128_64,
                    new ClickableButton.CallbackHandler() {
                        @Override
                        public void onButtonPressed() {
                            if(MutateLabWindow.this.callback.onPartsChosen(HeadBodyPart.Type.valueOf(head[currentHead].text),
                                                   TorsoBodyPart.Type.valueOf(torso[currentTorso].text),
                                                   LegsBodyPart.Type.valueOf(legs[currentLegs].text)) )
                            {
                                hide();
                            }
                        }
                    },
                    calculateX(),
                    calculateY(),
                    sizeX,
                    sizeY));
        
        this.uiElements.add(new WindowCooldownButton(Icon.DESTROY_NOBORDER,
                ButtonHorizontalAlignment.RIGHT,
                -2,
                ButtonVerticalAlignment.BOTTOM,
                -2,
                "",
                ButtonImageEnum.WINDOW_RECTANGLE_BUTTON_64_64,
                new WindowCooldownButton.Callback() {

                    @Override
                    public boolean onButtonPressed() {
                        MutateLabWindow.this.callback.onBuildingDestroy();
                        hide();
                        return true;
                    }
                },
                calculateX(), calculateY(),
                sizeX, sizeY, -1, new Tooltip(0, 0, "Destroy This Building", new String[] {"WARNING! You'll get no refund."}),
                callback.getTooltipPlayer()));
    }

    protected int changeIndex(int lenght, int current, int amount) {
        current += amount;
        if(current >= lenght) {
            current = 0;
        }
        if(current < 0) {
            current = lenght - 1;
        }
        
        return current;
    }
    
    @Override
    protected void windowTick(float mousePosX, float mousePosY) {
    }

    @Override
    protected void windowRender(int windowX, int windowY, int sizeX, int sizeY) {
        totalCost.updateCoords(windowX, windowY, sizeX, sizeY);
        
        CostWrapper costHead = HeadBodyPart.Type.valueOf(head[currentHead].text).getCost();
        CostWrapper costTorso = TorsoBodyPart.Type.valueOf(torso[currentTorso].text).getCost();
        CostWrapper costLegs = LegsBodyPart.Type.valueOf(legs[currentLegs].text).getCost();
        
        int coinsAmount = costHead.coinsAmount + costTorso.coinsAmount + costLegs.coinsAmount;
        int bonusAmount = costHead.bonusAmount + costTorso.bonusAmount + costLegs.bonusAmount;
        
        BitmapFont.TextBounds textSize = DrawableData.blackFont.getBounds(coinsAmount + "");
        float coinsSizeX = textSize.width;
        textSize = DrawableData.blackFont.getBounds(bonusAmount + "");
        float bonusSizeX = textSize.width;
        textSize = DrawableData.blackFont.getBounds("Total cost: ");
        float textSizeX = textSize.width;
        
        float totalSizeX = textSizeX;
        if(coinsAmount > 0) {
            totalSizeX += coinsSizeX + 33;
        }
        if(bonusAmount > 0) {
            totalSizeX += bonusSizeX + 13 + 16;
        }
        
        int startX = (int)(totalCost.calculateX() - totalSizeX/2.0f);
        int startY = totalCost.calculateY();
        
        if(coinsAmount > 0 || bonusAmount > 0) {
            RenderUtils.blitText(DrawableData.spriteBatch,
                        "Total cost: ",
                        startX,
                        startY,
                        RenderUtils.TextAlligment.LEFT,
                        RenderUtils.TextAlligment.TOP);
            
            startX += textSizeX;
            
            if(coinsAmount > 0) {
                RenderUtils.blitText(DrawableData.spriteBatch,
                        coinsAmount + "",
                        startX,
                        startY,
                        RenderUtils.TextAlligment.LEFT,
                        RenderUtils.TextAlligment.TOP);

                startX = (int)(startX + coinsSizeX) + 16;
                Icon.COIN.render(startX, startY - 8, 24, 24);
                startX += 16;
            }

            if(bonusAmount > 0) {
                RenderUtils.blitText(DrawableData.spriteBatch,
                        bonusAmount + "",
                        startX,
                        startY,
                        RenderUtils.TextAlligment.LEFT,
                        RenderUtils.TextAlligment.TOP);

                startX = (int)(startX + bonusSizeX) + 16;
                Icon.TOFU.render(startX, startY - 8, 24, 24);
            }
        }
    }
}
