package com.pancik.libgdx.zumpatalentov.engine.ui.window;

import com.badlogic.gdx.graphics.Texture;
import com.pancik.libgdx.zumpatalentov.DrawableData;
import com.pancik.libgdx.zumpatalentov.ui.ButtonHorizontalAlignment;
import com.pancik.libgdx.zumpatalentov.ui.ButtonImageEnum;
import com.pancik.libgdx.zumpatalentov.ui.ButtonVerticalAlignment;
import com.pancik.libgdx.zumpatalentov.ui.ClickableButton;
import com.pancik.libgdx.zumpatalentov.util.MouseEvent;
import com.pancik.libgdx.zumpatalentov.util.RenderUtils;
import java.util.ArrayList;
import java.util.List;

public abstract class Window {
    public static Texture background = DrawableData.textures.get("window_background");
    
    protected ButtonHorizontalAlignment horizontal;
    protected int deltaX;
    protected ButtonVerticalAlignment vertical;
    protected int deltaY;
    protected int sizeX;
    protected int sizeY;
    
    protected boolean hidden = false;
    protected List<WindowUIElement> uiElements = new ArrayList<WindowUIElement>();

    public Window(ButtonHorizontalAlignment horizontal, int deltaX, ButtonVerticalAlignment vertical, int deltaY, int sizeX, int sizeY) {
        this.horizontal = horizontal;
        this.deltaX = deltaX;
        this.vertical = vertical;
        this.deltaY = deltaY;
        this.sizeX = sizeX;
        this.sizeY = sizeY;
        
        this.uiElements.add(new WindowButton(ButtonHorizontalAlignment.RIGHT,
                -4,
                ButtonVerticalAlignment.TOP,
                5,
                "",
                ButtonImageEnum.WINDOW_X_BUTTON_32_32,
                new ClickableButton.CallbackHandler() {

                    @Override
                    public void onButtonPressed() {
                        hide();
                    }
                },
                calculateX(),
                calculateY(),
                sizeX,
                sizeY));
    }
    
    public void tick(float mousePosX, float mousePosY) {
        for(WindowUIElement element : uiElements) {
            element.tick(mousePosX, mousePosY, hidden);
        }

        windowTick(mousePosX, mousePosY);
    }
    
    public void renderUI() {
        if(!hidden) {
            int x = calculateX();
            int y = calculateY();

            RenderUtils.blit(DrawableData.spriteBatch,
                    background,
                    new RenderUtils.ScreenInfo(x, y, sizeX, sizeY),
                    new RenderUtils.ScreenInfo(0, 0, 1024, 1024));

            for(WindowUIElement element : uiElements) {
                element.render(x, y, sizeX, sizeY);
            }

            windowRender(x, y, sizeX, sizeY);
        }
    }
    
    public boolean handle(MouseEvent me) {
        if(!hidden) {
            for(WindowUIElement element : uiElements) {
                element.handle(me);
            }
            
            return (hitsWindow(me.getUiUnprojectedX(), me.getUiUnprojectedY()));
        } else {
            return false;
        }
    }

    public void show() {
        hidden = false;
    }
    
    public void hide() {
        hidden = true;
    }
    
    public boolean isHidden() {
        return hidden;
    }
    
    public final int calculateX() {
        return RenderUtils.calculateX(horizontal, deltaX, sizeX);
    }
    
    public final int calculateY() {
        return RenderUtils.calculateY(vertical, deltaY, sizeY);
    }
    
    public boolean hitsWindow(int x, int y) {
        if (calculateX() < x && calculateX() + sizeX > x && calculateY() < y && calculateY() + sizeY > y) {
            return true;
        } else {
            return false;
        }
    }
    
    protected abstract void windowTick(float mousePosX, float mousePosY);
    protected abstract void windowRender(int windowX, int windowY, int sizeX, int sizeY);
}
