package com.pancik.libgdx.zumpatalentov.engine.ui.window;

import com.pancik.libgdx.zumpatalentov.DrawableData;
import com.pancik.libgdx.zumpatalentov.ui.ButtonHorizontalAlignment;
import com.pancik.libgdx.zumpatalentov.ui.ButtonImageEnum;
import com.pancik.libgdx.zumpatalentov.ui.ButtonVerticalAlignment;
import com.pancik.libgdx.zumpatalentov.ui.ClickableButton;

public class WindowButton extends ClickableButton implements WindowUIElement {

    protected int offsetX;
    protected int offsetY;
    protected int sizeX;
    protected int sizeY;
    
    public WindowButton(ButtonHorizontalAlignment horizontal, int deltax,
            ButtonVerticalAlignment vertical, int deltay,
            String text,
            ButtonImageEnum imageType,
            CallbackHandler callback,
            int offsetX,
            int offsetY,
            int sizeX,
            int sizeY) {
        super(horizontal, deltax, vertical, deltay, text, imageType, callback, false);
        this.offsetX = offsetX;
        this.offsetY = offsetY;
        this.sizeX = sizeX;
        this.sizeY = sizeY;
    }
    
    @Override
    public void tick(float mousePosX, float mousePosY, boolean hidden) {
    }

    @Override
    public void render(int x, int y, int sizeX, int sizeY) {
        this.offsetX = x;
        this.offsetY = y;
        this.sizeX = sizeX;
        this.sizeY = sizeY;
        
        render();
    }

    @Override
    protected int calculateX() {
        int width = sizeX;

        switch(horizontal) {
            case LEFT:
                return offsetX + (int)(deltax * DrawableData.getDensity());
            case MIDDLE:
                return offsetX + (width/2) + (int)(deltax * DrawableData.getDensity()) - imageType.getDensitySizex()/2;
            case RIGHT:
                return offsetX + width + (int)(deltax * DrawableData.getDensity()) - imageType.getDensitySizey();
        }
        return -1;
    }

    @Override
    protected int calculateY() {
        int height = sizeY;

        switch(vertical) {
            case TOP:
                return offsetY + (int)(deltay * DrawableData.getDensity());
            case MIDDLE:
                return offsetY + (height/2) + (int)(deltay * DrawableData.getDensity()) - imageType.getDensitySizey()/2;
            case BOTTOM:
                return offsetY + height + (int)(deltay * DrawableData.getDensity())  - imageType.getDensitySizey();
        }
        return -1;
    }
}
