package com.pancik.libgdx.zumpatalentov.engine.ui.window;

import com.badlogic.gdx.graphics.Texture;
import com.pancik.libgdx.zumpatalentov.DrawableData;
import com.pancik.libgdx.zumpatalentov.engine.player.Player;
import com.pancik.libgdx.zumpatalentov.engine.ui.Icon;
import com.pancik.libgdx.zumpatalentov.engine.ui.Tooltip;
import com.pancik.libgdx.zumpatalentov.ui.ButtonHorizontalAlignment;
import com.pancik.libgdx.zumpatalentov.ui.ButtonVerticalAlignment;
import com.pancik.libgdx.zumpatalentov.util.MouseEvent;
import com.pancik.libgdx.zumpatalentov.util.RenderUtils;

public class WindowWellIcon implements WindowUIElement {
    public static Texture wellTexture = DrawableData.textures.get("window_well");
    public static final int TEX_SIZE_X = 128;
    public static final int TEX_SIZE_Y = 128;
    
    protected Icon icon;
    protected Tooltip tooltip;
    protected Player tooltipPlayer;
    protected ButtonHorizontalAlignment horizontal;
    protected int deltaX;
    protected ButtonVerticalAlignment vertical;
    protected int deltaY;
    protected int sizeX;
    protected int sizeY;

    protected int offsetX;
    protected int offsetY;
    protected int windowSizeX;
    protected int windowSizeY;
    
    public WindowWellIcon(Icon icon, Tooltip tooltip, Player tooltipPlayer, ButtonHorizontalAlignment horizontal, int deltaX, ButtonVerticalAlignment vertical, int deltaY, int sizeX, int sizeY) {
        this.icon = icon;
        this.tooltip = tooltip;
        this.tooltipPlayer = tooltipPlayer;
        this.horizontal = horizontal;
        this.deltaX = deltaX;
        this.vertical = vertical;
        this.deltaY = deltaY;
        
        this.sizeX = sizeX;
        this.sizeY = sizeY;
    }
    
    @Override
    public void tick(float mousePosX, float mousePosY, boolean hidden) {
        if(hitsWell((int)mousePosX, (int)mousePosY) && !hidden) {
            tooltip.setPos(mousePosX, mousePosY);
            tooltipPlayer.setTooltip(tooltip);
        }
    }

    @Override
    public void render(int x, int y, int windowSizeX, int windowSizeY) {
        this.offsetX = x;
        this.offsetY = y;
        this.windowSizeX = windowSizeX;
        this.windowSizeY = windowSizeY;
        
        RenderUtils.blit(DrawableData.spriteBatch,
                wellTexture,
                new RenderUtils.ScreenInfo(calculateX(), calculateY(), sizeX, sizeY),
                new RenderUtils.ScreenInfo(0, 0, TEX_SIZE_X, TEX_SIZE_Y));
        
        icon.render(calculateX() + sizeX/2, calculateY() + sizeY/2, sizeX, sizeY);
    }

    public boolean hitsWell(int posx, int posy) {
        if(posx >= calculateX() && posx <= calculateX() + sizeX && posy >= calculateY() && posy <= calculateY() + sizeY) {
            return true;
        } else {
            return false;
        }
    }
    
    @Override
    public void handle(MouseEvent me) {
    }
    
    public void set(Icon icon, Tooltip tooltip) {
        this.icon = icon;
        this.tooltip = tooltip;
    }
    
    protected int calculateX() {
        int width = windowSizeX;

        switch(horizontal) {
            case LEFT:
                return offsetX + (int)(deltaX * DrawableData.getDensity());
            case MIDDLE:
                return offsetX + (width/2) + (int)(deltaX * DrawableData.getDensity()) - sizeX/2;
            case RIGHT:
                return offsetX + width + (int)(deltaX * DrawableData.getDensity()) - sizeX;
        }
        return -1;
    }

    protected int calculateY() {
        int height = windowSizeY;

        switch(vertical) {
            case TOP:
                return offsetY + (int)(deltaY * DrawableData.getDensity());
            case MIDDLE:
                return offsetY + (height/2) + (int)(deltaY * DrawableData.getDensity()) - sizeY/2;
            case BOTTOM:
                return offsetY + height + (int)(deltaY * DrawableData.getDensity()) - sizeY;
        }
        return -1;
    }
}
