package com.pancik.libgdx.zumpatalentov.entity;

import com.badlogic.gdx.math.Vector2;
import com.pancik.libgdx.zumpatalentov.engine.Level;
import com.pancik.libgdx.zumpatalentov.entity.unit.UnitEntity;
import com.pancik.libgdx.zumpatalentov.entity.unit.UnitEntity.MoveResult;
import com.pancik.libgdx.zumpatalentov.entity.unit.UnitEntity.MoveType;
import com.pancik.libgdx.zumpatalentov.util.RandomGenerator;

public class WaypointAI implements UnitEntityAI {
    public static final float TOLERATED_MISS = 64.0f;
    public static final float WANDER_DISTANCE = 128.0f;
    
    private Level.WaypointPath waypointPath;
    private int currentWaypoint = 0;

    private int chosenTypeStep = 0;
    private int chosenTypeSelectedStepsSize = 0;
    private final int chosenTypeMaxSteps = 30;
    private MoveType chosenType = null;
    
    private final int wanderLength = 30;
    private int wanderCooldownStep = 0;
    private int wanderLengthRemaining = wanderLength;
    private final int wanderCooldown = 180;
    private Vector2 wanderTarget = null;
    
    @Override
    public void tick(UnitEntity e) {
        if(currentWaypoint < waypointPath.getWaypoints().size()) {
            Vector2 target = waypointPath.getWaypoints().get(currentWaypoint).cpy();
            goToTarget(e, target);
        } else {
            //wander
            if(wanderLengthRemaining > 0) {
                if(wanderTarget == null) {
                    Vector2 target = new Vector2(0.0f, -WANDER_DISTANCE);
                    float[] angles = {0.0f, 45.0f, 90.0f, 135.0f, 180.0f, 225.0f, 270.0f, 315.0f};
                    target.rotate(angles[RandomGenerator.getRandom(angles.length)]);
                    target.add(e.getPos());
                    wanderTarget = target;
                }
                
                goToTarget(e, wanderTarget);
                wanderLengthRemaining--;
            } else {
                wanderTarget = null;
                
                wanderCooldownStep++;
                if(wanderCooldownStep >= wanderCooldown){
                    wanderCooldownStep = 0;
                    wanderLengthRemaining = wanderLength/2 + RandomGenerator.getRandom(wanderLength/2);
                }
            }
        }
    }
    
    protected void goToTarget(UnitEntity e, Vector2 target) {
        if (chosenType == null) {
            //System.out.println("target: " + target);

            MoveResult[] results = e.getMoveResults();

            int closestOne = 0;
            float length = 1000000.0f;
            for (int i = 0; i < results.length; i++) {
                Vector2 pos = results[i].result;
                pos.sub(target);

                if (pos.len() < length) {
                    closestOne = i;
                    length = pos.len();
                }
            }

            if(results.length > closestOne) {
                if(results[closestOne] != null) {
                    e.apply(results[closestOne].move);

                    chosenType = results[closestOne].move;
                    chosenTypeStep = 0;
                    chosenTypeSelectedStepsSize = 15 + RandomGenerator.getRandom(chosenTypeMaxSteps - 15);
                }
            } else {
                //System.out.println("closest one: " + closestOne + " lenght: " + results.length);
            }
        } else {
            e.apply(chosenType);
            chosenTypeStep++;
            if (chosenTypeStep >= chosenTypeSelectedStepsSize) {
                chosenType = null;
            }
        }
        
        checkWaypointDistance(e);
    }
    
    protected void checkWaypointDistance(UnitEntity e) {
        if(currentWaypoint < waypointPath.getWaypoints().size()) {
            Vector2 target = waypointPath.getWaypoints().get(currentWaypoint).cpy();

            Vector2 pos = e.getPos();
            pos.sub(target);
            if (pos.len() < TOLERATED_MISS) {
                currentWaypoint++;
            }
        }
    }
    
    public void setWaypointPath(Level.WaypointPath path) {
        this.waypointPath = path;
    }
}
