package com.pancik.libgdx.zumpatalentov.entity.unit.bodypart;

import com.badlogic.gdx.Gdx;
import com.badlogic.gdx.graphics.Texture;
import com.badlogic.gdx.math.Vector2;
import com.pancik.libgdx.zumpatalentov.DrawableData;
import com.pancik.libgdx.zumpatalentov.SoundData;
import com.pancik.libgdx.zumpatalentov.engine.DelayedAction;
import com.pancik.libgdx.zumpatalentov.engine.Engine.EngineControls;
import com.pancik.libgdx.zumpatalentov.entity.attackable.AttackableEntity;
import com.pancik.libgdx.zumpatalentov.entity.projectile.PlasmaHomingProjectile;
import com.pancik.libgdx.zumpatalentov.entity.projectile.Projectile;
import com.pancik.libgdx.zumpatalentov.entity.unit.UnitEntity;
import com.pancik.libgdx.zumpatalentov.entity.unit.buff.Buff;
import com.pancik.libgdx.zumpatalentov.util.RenderUtils;

public class TwoHandTorso extends TorsoBodyPart {
    public static Texture TORSO_MELEE_MOVE = DrawableData.textures.get("twohand_torso_move");
    public static Texture TORSO_MELEE_ATTACK = DrawableData.textures.get("twohand_torso_attack");
    
    protected static final int[] ANIMATION_STEPS = {0,0,0,0,0,0,0, 1,1,1,1,1,1,1, 2,2,2,2,2,2,2, 3,3,3,3,3,3,3,
                                                    4,4,4,4,4,4,4, 5,5,5,5,5,5,5, 6,6,6,6,6,6,6, 7,7,7,7,7,7,7};
    
    protected static final int[] ATTACK_STEPS = {0,0,0,0,0,0,0, 1,1,1,1,1,1,1, 2,2,2,2,2,2,2, 3,3,3,3,3,3,3,
                                                 0,0,0,0,0,0,0};
    
    public static final float ATTACK_SIGHT_RANGE = 350.0f;
    public static final float ATTACK_RANGE = 300.0f;
    public static final int ATTACK_DAMAGE = 25;
    public static final int COOLDOWN_TIME = 60;

    public TwoHandTorso(EngineControls engineControls) {
        super(engineControls);
    }
 
    @Override
    public boolean attack(final AttackableEntity defendingEntity, final UnitEntity attackingEntity, final Vector2 sidePos) {
        if (defendingEntity.getPos().dst(attackingEntity.getPos()) <= getAttackRange()) {
            SoundData.playSound("laser_rifle", 0.5f);
            
            engineControls.addDelayedAction(new DelayedAction(25, new DelayedAction.Callback() {

                @Override
                public void onDelayEnded() {
                    engineControls.addEntity(new PlasmaHomingProjectile(defendingEntity, sidePos,new Projectile.Callback() {
                        @Override
                        public void onProjectileArrived() {
                            defendingEntity.getHit((int)(ATTACK_DAMAGE * attackingEntity.getBuffCoefficient(Buff.PercentageStat.ATTACK_POWER)), attackingEntity);
                        }
                    }, engineControls));
                }
            }));
            return true;
        } else {
            return false;
        }
    }

    @Override
    public void tick() {
    }

    @Override
    public void render(Vector2 pos, Vector2 size, int animationStep, int animationDirection, boolean attacked) {
        if(!attacked) {
            RenderUtils.blit(DrawableData.spriteBatch,
                    TORSO_MELEE_MOVE,
                    new RenderUtils.ScreenInfo(pos.x - size.x/2, pos.y - size.y/2, size.x, size.y),
                    new RenderUtils.ScreenInfo(TEX_SIZE_X * ANIMATION_STEPS[animationStep % ANIMATION_STEPS.length], TEX_SIZE_Y * animationDirection, TEX_SIZE_X, TEX_SIZE_Y));
        } else {
            if(animationStep < ATTACK_STEPS.length) {
                RenderUtils.blit(DrawableData.spriteBatch,
                        TORSO_MELEE_ATTACK,
                        new RenderUtils.ScreenInfo(pos.x - size.x/2, pos.y - size.y/2, size.x, size.y),
                        new RenderUtils.ScreenInfo(TEX_SIZE_X * ATTACK_STEPS[animationStep], TEX_SIZE_Y * animationDirection, TEX_SIZE_X, TEX_SIZE_Y));
            } else {
                RenderUtils.blit(DrawableData.spriteBatch,
                        TORSO_MELEE_ATTACK,
                        new RenderUtils.ScreenInfo(pos.x - size.x/2, pos.y - size.y/2, size.x, size.y),
                        new RenderUtils.ScreenInfo(TEX_SIZE_X * ATTACK_STEPS[ATTACK_STEPS.length - 1], TEX_SIZE_Y * animationDirection, TEX_SIZE_X, TEX_SIZE_Y));
            }
        }

        /*
        RenderUtils.blit(DrawableData.spriteBatch,
                    GrassTile.GRASS_TILE_TEXTURE,
                    new RenderUtils.ScreenInfo(pos.x - 5, pos.y - 5, 10, 10),
                    new RenderUtils.ScreenInfo(0, 0, 10, 10));
        */
    }

    @Override
    public int getAttackCooldown() {
        return COOLDOWN_TIME;
    }

    @Override
    public float getAttackRange() {
        return ATTACK_RANGE;
    }

    @Override
    public float getAttackSightRange() {
        return ATTACK_SIGHT_RANGE;
    }
}
