package com.pancik.libgdx.zumpatalentov.screens;

import com.badlogic.gdx.Gdx;
import com.pancik.libgdx.zumpatalentov.MainApplicationListener;
import com.pancik.libgdx.zumpatalentov.engine.Engine;
import com.pancik.libgdx.zumpatalentov.util.InputData;
import com.pancik.libgdx.zumpatalentov.util.KeyChangedEvent;
import com.pancik.libgdx.zumpatalentov.util.MouseEvent;
import com.pancik.libgdx.zumpatalentov.util.MouseScrolledEvent;

public class MainGameScreen implements Screen {
    private MainApplicationListener main;
    private Engine engine;

    public MainGameScreen(final MainApplicationListener main, String levelFile) {
        this.main = main;
        
        this.engine = new Engine(this.main, levelFile, new Engine.EngineCallback() {

            @Override
            public void hasWon() {
                //System.out.println("You have won the game.");
                main.setScreen(new SelectLevelScreen(main, true));
            }

            @Override
            public void hasLost() {
                //System.out.println("You have lost the game. Loser.");
                main.setScreen(new SelectLevelScreen(main, false));
            }
            
            @Override
            public void exited() {
                main.setScreen(new SelectLevelScreen(main));
            }
        });
    }

    @Override
    public void tick() {
        for(MouseEvent me : InputData.mouseEvents) {
            engine.handle(me);
        }
        for(MouseScrolledEvent mse : InputData.mouseScrolledEvents) {
            engine.handle(mse);
        }
        for(KeyChangedEvent ke : InputData.keyChangedEvents) {
            engine.handle(ke);
        }
        
        engine.tick();
    }

    @Override
    public void render() {
        engine.render();
    }

    @Override
    public void renderUI() {
        engine.renderUI();
    }

    @Override
    public void handleBackButtonPressed() {
        Gdx.app.exit();
    }

    @Override
    public void handleMenuButtonPressed() {
        //do nothing
    }
}